// 003 u_c_file_name
// 24 dec 2001

// -- (C) John COLIBRI 2001
// -- documentation: http://www.jcolibri.com

(*$r+*)

unit u_c_file_name;
  interface
    uses u_c_basic_object;

    type c_file_name= class(c_basic_object)
                        public
                          m_drive: string;
                          m_path: string;
                          m_name: string;
                          m_extension: string;

                          m_complete_name: string;

                          constructor create_file_name(p_name, p_drive, p_path, p_file_name, p_extension: string); Virtual;

                          procedure compute_complete_name;
                          procedure decompose;
                          procedure change_extension(p_extension: String);
                          procedure change_file_and_extension_name(p_file_and_extension_name: String);
                          procedure change_file_name_only(p_new_file_name: String);
                          procedure display_name_parts;
                          function f_drive_and_path: String;
                      end;

    function f_c_file_name(p_name, p_drive, p_path, p_file_name, p_extension: string): c_file_name;
    function f_c_complete_file_name(p_name, p_complete_name: string): c_file_name;

  implementation
    uses SysUtils, u_loaded, u_dir, u_c_display;

    procedure start;
      // do NOT remove: debug
      begin
      end; // start

    function c_file_name.f_drive_and_path: String;
      begin
        if m_path= ''
          then
            if m_drive= ''
              then Result:= ''
              else Result:= m_drive+ ':\'
          else
            if m_drive= ''
              then Result:= m_path
              else Result:= m_drive+ ':'+ f_with_beginning_and_ending_slash(m_path);
      end; // f_drive_and_path

    procedure c_file_name.compute_complete_name;
      begin
        if m_path= ''
          then begin
              if m_drive= ''
                then m_complete_name:= m_name+ '.'+ m_extension
                else m_complete_name:= m_drive+ ':\'+ m_name+ '.'+ m_extension;
            end
          else begin
              if m_drive= ''
                then m_complete_name:= f_with_ending_slash(m_path)+ m_name+ '.'+ m_extension
                else m_complete_name:= m_drive+ ':'+ f_with_beginning_and_ending_slash(m_path)+ m_name+ '.'+ m_extension;
            end;
      end; // compute_complete_name

    constructor c_file_name.create_file_name(p_name, p_drive, p_path, p_file_name, p_extension: string);
      begin
        Inherited create_basic_object(p_file_name);

        m_drive:= p_drive;
        m_path:= p_path;
        m_name:= p_file_name;
        m_extension:= p_extension;

        compute_complete_name;
      end; // create

    procedure c_file_name.display_name_parts;
      begin
        display('Dr '+ m_drive
          + ' path '+ m_path
          + ' name '+ m_name
          + ' ext '+ m_extension);
      end; // display_name_parts

    procedure c_file_name.decompose;
      var l_length, l_indice: Integer;
          l_path_length: Integer;

      procedure get_drive_or_dot;
        begin
          if (m_complete_name[1]= '.') and (m_complete_name[2]= '.')
            then begin
                m_path:= '..';
                l_path_length:= 2;

                l_indice:= 3;
              end
            else begin
                l_path_length:= 0;

                l_indice:= 1;

                // the drive
                if m_complete_name[2]= ':'
                  then begin
                      m_drive:= m_complete_name[1];
                      l_indice:= 3;
                    end
                  else begin
                      m_drive:= '';
                      l_indice:= 1;
                    end;
              end;
        end; // get_drive_or_dot

      procedure copie_max(p_indice_depart: Integer; var pv_chaine: String; p_taille, p_taille_max: Word);
        begin
          if p_taille> p_taille_max
            then p_taille:= p_taille_max;

          SetLength(pv_chaine, p_taille);
          move(m_complete_name[p_indice_depart], pv_chaine[1], p_taille);
        end; // copie_max

      procedure get_path;
        var l_start_index: integer;
            l_indice_dernier_slash: Integer;
        begin
          if m_complete_name[l_indice]= '\'
            then begin
                l_start_index:= l_indice;
                l_indice_dernier_slash:= l_indice;

                repeat
                  if m_complete_name[l_indice]= '\'
                    then l_indice_dernier_slash:= l_indice;
                  inc(l_indice);
                until l_indice>= l_length;

                // copy the path
                SetLength(m_path, l_indice_dernier_slash+ 1- l_start_index+ l_path_length);
                move(m_complete_name[l_start_index], m_path[1+ l_path_length], l_indice_dernier_slash+ 1- l_start_index);

                l_indice:= l_indice_dernier_slash+ 1;
              end;
          // display(IntToStr(l_indice)+ ' >'+ m_path+ '<');
        end; // get_path

      procedure get_name_and_extension;
        var l_start_index: integer;
            l_indice_point: integer;
        begin
          // looks for .
          l_start_index:= l_indice;
          l_indice_point:= 0;
          repeat
            // display(IntToStr(l_indice));
            if m_complete_name[l_indice]= '.'
              then l_indice_point:= l_indice;
            inc(l_indice);
          until l_indice>= l_length;

          // copy the name
          if l_indice_point> 0
            then begin
                m_name:= copy(m_complete_name, l_start_index, l_indice_point- l_start_index);
                m_extension:= copy(m_complete_name, l_indice_point+ 1, (l_length+ 1)- (l_indice_point+ 1));
              end
            else  m_name:= copy(m_complete_name, l_start_index, l_length+ 1- l_start_index);
        end; // get_name_and_extension

      begin // decompose
        // display(' >'+ m_complete_name+ '<');

        l_length:= length(m_complete_name);

        m_drive:= '';
        m_path:= '';
        m_name:= '';
        m_extension:= '';

        if l_length> 0
          then begin
              if l_length= 1
                then begin
                    m_name:= m_complete_name;
                  end
                else begin
                    get_drive_or_dot;
                    get_path;
                    get_name_and_extension;
                  end; // length > 1
            end; // length > 0
      end; // decompose

    procedure c_file_name.change_extension(p_extension: String);
      begin
        m_extension:= p_extension;
        compute_complete_name;
      end; // change_extension

    procedure c_file_name.change_file_and_extension_name(p_file_and_extension_name: String);
      begin
        m_complete_name:= m_drive+ ':\'+ f_with_ending_slash(m_path)+ p_file_and_extension_name;
        decompose;
      end; // change_file_and_extension_name

    procedure c_file_name.change_file_name_only(p_new_file_name: String);
      begin
        m_name:= p_new_file_name;
        compute_complete_name;
      end; // change_file_name_only

    // -- stand-alone procedures

    function f_c_file_name(p_name, p_drive, p_path, p_file_name, p_extension: string): c_file_name;
      begin
        Result:= c_file_name.create_file_name(p_name, p_drive, p_path, p_file_name, p_extension);
      end; // f_c_file_name

    function f_c_complete_file_name(p_name, p_complete_name: string): c_file_name;
      begin
        Result:= c_file_name.create_file_name(p_name, '', '', '', '');
        Result.m_complete_name:= p_complete_name;
        Result.decompose;
      end; // f_c_complete_file_name

    begin // u_file_name
      display_loaded('u_c_file_name', @start);
    end. // u_file_name

